"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LambdaLayerExternalsPlugin = void 0;
var path_1 = require("path");
var fs_1 = require("fs");
var webpack_1 = require("webpack");
var archiver = require("archiver");
var IGNORE_PACKAGES_REGEX = /^@aws-sdk[\\/]/;
var LambdaLayerExternalsPlugin = /** @class */ (function () {
    function LambdaLayerExternalsPlugin(options) {
        this.options = LambdaLayerExternalsPlugin.init(options);
        this.dependenciesMap = new Map();
        // this.skipPackages = this.getSkippablePackages();
        this.externalizedModules = new Set();
    }
    LambdaLayerExternalsPlugin.init = function (opt) {
        var _a, _b;
        return {
            moduleSource: (_a = opt === null || opt === void 0 ? void 0 : opt.moduleSource) !== null && _a !== void 0 ? _a : "./node_modules/",
            allowList: (_b = opt === null || opt === void 0 ? void 0 : opt.allowList) !== null && _b !== void 0 ? _b : [],
        };
    };
    LambdaLayerExternalsPlugin.prototype.apply = function (compiler) {
        var _this = this;
        compiler.hooks.compile.tap("LayerExternalsPlugin", function (_a) {
            var normalModuleFactory = _a.normalModuleFactory;
            normalModuleFactory.hooks.factorize.tapAsync("LayerExternalsPlugin", function (_a, callback) {
                var _b, _c;
                var request = _a.request;
                var type = (_b = _this.options.type) !== null && _b !== void 0 ? _b : (_c = compiler.options.output.library) === null || _c === void 0 ? void 0 : _c.type;
                return _this.externalizePackage(request)
                    ? callback(null, new webpack_1.ExternalModule(request, type))
                    : callback();
            });
        });
        compiler.hooks.afterEmit.tap("LayerExternalsPlugin", function (_a) {
            var _b;
            var outputOptions = _a.outputOptions;
            var moduleSource = _this.options.moduleSource;
            var sourceDir = (0, path_1.resolve)(moduleSource);
            var outputPath = (_b = outputOptions.path) !== null && _b !== void 0 ? _b : (0, path_1.resolve)("./dist/");
            LambdaLayerExternalsPlugin.writeToZip(sourceDir, outputPath, _this.externalizedModules);
        });
    };
    LambdaLayerExternalsPlugin.prototype.externalizePackage = function (request) {
        if (request.match(IGNORE_PACKAGES_REGEX))
            return true;
        this.updateDependenciesMap(request);
        if (this.dependenciesMap.has(request)) {
            this.externalizedModules.add(request);
            var dependencies = this.dependenciesMap.get(request);
            dependencies === null || dependencies === void 0 ? void 0 : dependencies.forEach(this.externalizePackage.bind(this));
            return true;
        }
        return false;
    };
    LambdaLayerExternalsPlugin.prototype.updateDependenciesMap = function (request) {
        var moduleSource = this.options.moduleSource;
        if (this.dependenciesMap.has(request))
            return;
        var packagePath = "".concat(moduleSource).concat(request);
        if ((0, fs_1.existsSync)(packagePath))
            this.dependenciesMap.set(request, this.getPackageDependencies(request));
    };
    // private getSkippablePackages(): Packages {
    //   const { allowList } = this.options;
    //
    //   return allowList.reduce((set: Packages, allowedItem) => {
    //     const { packageName, allowSubPackages } = allowedItem;
    //
    //     set.add(packageName);
    //
    //     if (allowSubPackages ?? true) {
    //       this.getPackageDependencies(packageName).forEach((subPackage) => {
    //         set.add(subPackage);
    //       });
    //     }
    //
    //     return set;
    //   }, new Set<string>());
    // }
    LambdaLayerExternalsPlugin.prototype.getPackageDependencies = function (packageName) {
        var moduleSource = this.options.moduleSource;
        var packagePath = "".concat(moduleSource).concat(packageName, "/package.json");
        var dependencies = JSON.parse((0, fs_1.readFileSync)(packagePath, "utf8")).dependencies;
        return dependencies ? new Set(Object.keys(dependencies)) : new Set();
    };
    LambdaLayerExternalsPlugin.writeToZip = function (modulePath, outputPath, externalizedModules) {
        var output = (0, fs_1.createWriteStream)("".concat(outputPath, "/externals.zip"));
        var archive = archiver("zip", {
            zlib: { level: 9 },
        });
        archive.pipe(output);
        externalizedModules.forEach(function (module) {
            archive.directory("".concat(modulePath, "/").concat(module), "nodejs/node_modules/".concat(module));
        });
        archive.finalize();
    };
    return LambdaLayerExternalsPlugin;
}());
exports.LambdaLayerExternalsPlugin = LambdaLayerExternalsPlugin;
